"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RemoteLogger = exports.RemoteLoggerEventTypes = void 0;
const core_1 = require("@bifold/core");
const react_native_1 = require("react-native");
const react_native_logs_1 = require("react-native-logs");
const transports_1 = require("./transports");
var RemoteLoggerEventTypes;
(function (RemoteLoggerEventTypes) {
    RemoteLoggerEventTypes["ENABLE_REMOTE_LOGGING"] = "RemoteLogging.Enable";
})(RemoteLoggerEventTypes || (exports.RemoteLoggerEventTypes = RemoteLoggerEventTypes = {}));
class RemoteLogger extends core_1.BifoldLogger {
    constructor(options) {
        var _a, _b, _c;
        super();
        this._remoteLoggingEnabled = false;
        this._autoDisableRemoteLoggingIntervalInMinutes = 0;
        this._config = {
            levels: {
                test: 0,
                trace: 0,
                debug: 0,
                info: 1,
                warn: 2,
                error: 3,
                fatal: 4,
            },
            severity: 'debug',
            async: true,
            dateFormat: 'time',
            printDate: false,
        };
        this.lokiUrl = (_a = options.lokiUrl) !== null && _a !== void 0 ? _a : undefined;
        this.lokiLabels = (_b = options.lokiLabels) !== null && _b !== void 0 ? _b : {};
        this._autoDisableRemoteLoggingIntervalInMinutes = (_c = options.autoDisableRemoteLoggingIntervalInMinutes) !== null && _c !== void 0 ? _c : 0;
        this.configureLogger();
    }
    get sessionId() {
        if (!this._sessionId) {
            this._sessionId = Math.floor(100000 + Math.random() * 900000);
        }
        return this._sessionId;
    }
    set sessionId(value) {
        this._sessionId = value;
        this.configureLogger();
    }
    get autoDisableRemoteLoggingIntervalInMinutes() {
        return this._autoDisableRemoteLoggingIntervalInMinutes;
    }
    get remoteLoggingEnabled() {
        return this._remoteLoggingEnabled;
    }
    set remoteLoggingEnabled(value) {
        this._remoteLoggingEnabled = value;
        if (value === false) {
            this._sessionId = undefined;
        }
        this.configureLogger();
    }
    configureLogger() {
        const transportOptions = {};
        const transport = [react_native_logs_1.consoleTransport];
        const config = Object.assign(Object.assign({}, this._config), { transport,
            transportOptions });
        if (this.remoteLoggingEnabled && this.lokiUrl) {
            transport.push(transports_1.lokiTransport);
            config['transportOptions'] = {
                lokiUrl: this.lokiUrl,
                lokiLabels: Object.assign(Object.assign({}, this.lokiLabels), { session_id: `${this.sessionId}` }),
            };
            if (this.autoDisableRemoteLoggingIntervalInMinutes && this.autoDisableRemoteLoggingIntervalInMinutes > 0) {
                this.remoteLoggingAutoDisableTimer = setTimeout(() => {
                    this.remoteLoggingEnabled = false;
                }, this.autoDisableRemoteLoggingIntervalInMinutes * 60000);
            }
        }
        this._log = react_native_logs_1.logger.createLogger(config);
    }
    startEventListeners() {
        this.eventListener = react_native_1.DeviceEventEmitter.addListener(RemoteLoggerEventTypes.ENABLE_REMOTE_LOGGING, (value) => {
            this.remoteLoggingEnabled = value;
        });
    }
    stopEventListeners() {
        this.eventListener = undefined;
    }
    overrideCurrentAutoDisableExpiration(expirationInMinutes) {
        if (expirationInMinutes <= 0) {
            return;
        }
        if (this.remoteLoggingAutoDisableTimer) {
            clearTimeout(this.remoteLoggingAutoDisableTimer);
        }
        this.remoteLoggingAutoDisableTimer = setTimeout(() => {
            this.remoteLoggingEnabled = false;
        }, expirationInMinutes * 60000);
    }
    test(message, data) {
        var _a;
        (_a = this._log) === null || _a === void 0 ? void 0 : _a.test({ message, data });
    }
    trace(message, data) {
        var _a;
        (_a = this._log) === null || _a === void 0 ? void 0 : _a.trace({ message, data });
    }
    debug(message, data) {
        var _a;
        (_a = this._log) === null || _a === void 0 ? void 0 : _a.debug({ message, data });
    }
    info(message, data) {
        var _a;
        (_a = this._log) === null || _a === void 0 ? void 0 : _a.info({ message, data });
    }
    warn(message, data) {
        var _a;
        (_a = this._log) === null || _a === void 0 ? void 0 : _a.warn({ message, data });
    }
    error(message, data) {
        var _a;
        (_a = this._log) === null || _a === void 0 ? void 0 : _a.error({ message, data });
    }
    fatal(message, data) {
        var _a;
        (_a = this._log) === null || _a === void 0 ? void 0 : _a.fatal({ message, data });
    }
    report(bifoldError) {
        var _a;
        (_a = this._log) === null || _a === void 0 ? void 0 : _a.info({ message: 'Sending Loki report' });
        const { title, description, code, message } = bifoldError;
        (0, transports_1.lokiTransport)({
            msg: title,
            rawMsg: [{ message: title, data: { title, description, code, message } }],
            level: { severity: 3, text: 'error' },
            options: {
                lokiUrl: this.lokiUrl,
                lokiLabels: this.lokiLabels,
                job: 'incident-report',
            },
        });
    }
}
exports.RemoteLogger = RemoteLogger;
