"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.lokiTransport = void 0;
/* eslint-disable no-console */
const axios_1 = __importDefault(require("axios"));
const buffer_1 = require("buffer");
const lokiTransport = (props) => {
    var _a;
    // Loki requires a timestamp with nanosecond precision
    // however Date.now() only returns milliseconds precision.
    const timestampEndPadding = '000000';
    if (!props.options) {
        throw Error('props.options is required');
    }
    if (!props.options.lokiUrl) {
        throw Error('props.options.lokiUrl is required');
    }
    if (!props.options.lokiLabels) {
        throw Error('props.options.labels is required');
    }
    const { lokiUrl, lokiLabels } = props.options;
    const { message, data } = props.rawMsg.pop();
    const payload = {
        streams: [
            {
                stream: Object.assign({ job: (_a = props.options.job) !== null && _a !== void 0 ? _a : 'react-native-logs', level: props.level.text }, lokiLabels),
                values: [[`${Date.now()}${timestampEndPadding}`, JSON.stringify({ message, data })]],
            },
        ],
    };
    const [credentials, href] = lokiUrl.split('@');
    const [username, password] = credentials.split('//')[1].split(':');
    const protocol = credentials.split('//')[0];
    const authHeader = `Basic ${buffer_1.Buffer.from(`${username}:${password}`).toString('base64')}`;
    const config = {
        headers: {
            'Content-Type': 'application/json',
            Authorization: authHeader,
        },
    };
    axios_1.default
        .post(`${protocol}//${href}`, payload, config)
        .then((res) => {
        if (res.status !== 204) {
            console.warn(`Expected Loki to return 204, received ${res.status}`);
        }
    })
        .catch((error) => {
        console.error(`Error while sending to Loki, ${error}`);
    });
};
exports.lokiTransport = lokiTransport;
